"""Like the input built-in, but with bells and whistles."""

import getpass

# Use raw_input for Python 2.x
try:
    input = raw_input
except NameError:
    pass

def user_input(field, default='', choices=None, password=False, empty_ok=False, accept=False):
    """Prompt user for input until a value is retrieved or default
    is accepted. Return the input.

    Arguments:
    
    *field* - Description of the input being prompted for.

    *default* - Default value for the input accepted with a Return-key.

    *password* - Whether the user input should not be echoed to screen.

    *empty_ok* - Whether it's okay to accept an empty input.

    *accept* - Whether to skip getting actual user input and just accept
    the default value, unless prevented by the combination of
    arguments *empty_ok* and *default*. That is, unless *default* 
    is an empty string and *empty_ok* is False.
    """
    result = ''
    while not result:
        prompt = field
        if default:
            prompt += ' [{0}]'.format(default)
        prompt += ': '
        if accept and not (not default and not empty_ok):
            print(prompt)
            result = '{0}'.format(default)
        else:
            if password:
                result = getpass.getpass(prompt)
            else:
                result = input(prompt)
        result = result.strip()
        if not result:
            result = default
        if choices and result not in choices:
            print('Must be one of {0}'.format(choices))
            result = ''
        if empty_ok:
            break
    return result



__version__ = '1.0.5'
from .Averager import Averager
from .Config import Config
from .MapSock import MapSockServer, MapSockClient, MapSockRequest
from .RateTicker import RateTicker
from .Ring import Ring
from .SocketTalk import SocketTalk
from .SortedList import SortedList
from .String import string2time, time2string, time2levels, time2dir, time2fname
from .Timer import Timer
from .UserInput import user_input



"""Defines SortedList class."""

import bisect

class SortedList(object):
    """Maintains a list of sorted items, with fast trimming
    using less-than/greater-than comparison."""
    
    def __init__(self, donor=list()):
        """Initialize the object with a copy of the donor list, sorted."""
        self._list = sorted(donor[:])
        
    def add(self, item):
        """Add item to the list while maintaining sorted order."""
        #
        # Native list append() should work, but
        # only if we guaranteed that we're adding the "greatest" item.
        #
        #self._list.append(item)
        #
        # But instead we'll use the sorted insertion.
        bisect.insort_left(self._list, item)

    def getCountLT(self, item):
        """Return number of elements less than *item*."""
        index = bisect.bisect_left(self._list, item)
        return index

    def getCountGT(self, item):
        """Return number of elements greater than *item*."""
        index = bisect.bisect_right(self._list, item)
        return len(self._list) - index

    def removeLT(self, item):
        """Trim off any elements less than *item*.
        Return number of elements trimmed."""
        count = self.getCountLT(item)
        self._list = self._list[count:]
        return count



def test1():
    """Start from empty."""
    from SortedList import SortedList
    a = SortedList()
    for val in range(10):
        a.add(val)
    assert a.getCountLT(5) == 5
    assert a.getCountGT(5) == 4
    assert a.removeLT(5) == 5
    assert a.getCountLT(5) == 0
test1()

def test2():
    """Construct from list."""
    from SortedList import SortedList
    a = SortedList([b for b in range(10)])
    assert a.getCountLT(5) == 5
    assert a.getCountGT(5) == 4
    assert a.removeLT(5) == 5
    assert a.getCountLT(5) == 0
test2()



"""Defines RateTicker class."""

import datetime

# Keep Python 2 and 3 happy during nosetests.
try:
    from SortedList import SortedList
except ImportError:
    from .SortedList import SortedList

class RateTicker(SortedList):
    """Computes rates of ticking."""
    
    def __init__(self, periods):
        """Initialize the object with a tuple of time periods in seconds.
        For example, use (60, 300, 900) to track rates at 1, 5 and 15 minute
        periods (like when reporting system load.)"""
        super(RateTicker, self).__init__()
        self._periods = periods

    def tick(self):
        """Tick the ticker. 
        Return a tuple of values corresponding to periods given in initializer, 
        each value representing the rate of ticks (number of ticks per second)
        during that period."""
        now = datetime.datetime.now()
        self.add(now)

        # Create a list of timestamps, one for each period and 
        # representing the beginning of that period (i.e. since now.)
        tstamps = [now - datetime.timedelta(seconds=xx) for xx in self._periods]
        
        # Trim off any tick values older than the earliest timestamp.
        self.removeLT(min(tstamps))

        # Create a list of counts, one for each period and 
        # representing the number of ticks in that period.
        counts = [self.getCountGT(xx) for xx in tstamps]

        # Compute a list of rates for the periods.
        rates = [float(xx)/yy for xx,yy in zip(counts, self._periods)]

        # Return the rates as a tuple.
        return tuple(rates)



def test000():
    """
    Hack to get nose to run tests properly.
    This file should be the first file processed by a nosetests run.
    """

    # Add parent directory to system path
    # as per answer http://stackoverflow.com/a/1054293
    # to question http://stackoverflow.com/q/1054271 .
    import sys
    import os
    sys.path.append(
        os.path.abspath(
            os.path.join(
                os.path.dirname(__file__), 
                os.path.pardir,
                )
            )
        )
test000()



def test1():
    """Test the Ring data structure."""

    from Ring import Ring

    a = Ring([0,1,2])
    assert a[0] == 0
    assert a[1] == 1
    assert a[2] == 2
    assert a.first() == 0
    assert a.last() == 2

    a.turn()
    assert a[0] == 1
    assert a[1] == 2
    assert a[2] == 0
    assert a.first() == 1
    assert a.last() == 0

    a.turn()
    assert a[0] == 2
    assert a[1] == 0
    assert a[2] == 1
    assert a.first() == 2
    assert a.last() == 1
test1()



"""
A simple client/server framework with socket communication.
A server serves associative container of key/value pairs.
A client can insert/query/delete container elements.
"""

# Import system modules.
import logging
import socket
import pickle

# Import application modules.
import SocketTalk


class MapSockRequest(object):
    """
    Request object sent from client to server.
    """
    def __init__(self, action, key=None, value=None):
        self.action = action
        self.key = key
        self.value = value


class MapSockClient:
    """
    Client to the map server.
    """

    def __init__(self, host, port, encode=True):
        """
        Initialize the object.

        *host*  -  socket host

        *port*  -  socket port
        """
        self._logger = logging.getLogger(__name__)
        self._host = host
        self._port = port
        self._encode = encode

    def send(self, request):
        """
        Send request to server and return server response.
        """
        self._logger.debug('Opening connection')
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.connect((self._host, self._port))
        talk = SocketTalk.SocketTalk(sock, encode=self._encode)

        self._logger.debug('Sending action %s'%request.action)
        if not talk.put(request.action):
            self._logger.error('Failed to send action %s'%request.action)
            return None

        if request.key != None:
            self._logger.debug('Sending key %s'%request.key)
            if not talk.put(request.key):
                self._logger.error('Failed to send key %s'%request.key)
                return None

        if request.value != None:
            self._logger.debug('Sending value')
            if not talk.put(request.value):
                self._logger.error('Failed to send value')
                return None

        self._logger.debug('Receiving status')
        response = talk.get()
        if not response:
            self._logger.error('Failed to receive status')
            return None
        self._logger.debug('Status response = %s'%response)
        
        if request.action in ('get', 'size', 'keys') and response == 'ok':
            self._logger.debug('Receiving value')
            response = talk.get()
            if not response:
                self._logger.error('Failed to receive value')
                return None

        if request.action in ('keys',):
            response = pickle.loads(response)

        self._logger.debug('Closing connection')
        try:
            sock.shutdown(socket.SHUT_RDWR)
        except:
            self._logger.error('Failed to shutdown')
        sock.close()
        
        return response


class MapSockServer:
    def __init__(self, host, port, on_action=None, encode=True):
        """
        Initialize the server object.

        *host*       -  socket host

        *port*       -  socket port

        *on_action*  -  callback upon action reception, 
                        is called with action string
        """
        self._logger = logging.getLogger(__name__)
        self._data = dict()
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._sock.bind((host, port))
        self._sock.listen(1)
        self._talk = None
        self._on_action = on_action
        self._encode = encode

    def _send(self, message):
        """
        Return response message to client.
        """        
        result = self._talk.put(message)
        if not result:
            self._logger.error('Failed to send "%s"'%message)
        return result

    def _receive(self):
        """
        Receive a chunk of request from client.
        """
        result = self._talk.get()
        if not result:
            self._logger.error('Failed to receive')
        return result
            
    def run(self):
        """
        Continuously retrieve client requests until given "stop" request.
        """
        while True:
            self._logger.debug('Accepting connection')
            conn, addr = self._sock.accept()
            self._talk = SocketTalk.SocketTalk(conn, encode=self._encode)
            
            self._logger.debug('Receiving action')
            action = self._receive()
            
            if self._on_action:
                self._on_action(action)

            key = None
            if action in ('set', 'get', 'del',):
                self._logger.debug('Receiving key')
                key = self._receive()

            value = None
            if action in ('set',):
                self._logger.debug('Receiving value')
                value = self._receive()

            # Process the request.
            if action == 'stop':
                self._send('ok')

            elif action == 'set':
                self._data[key] = value
                self._send('ok')

            elif action == 'get':
                try:
                    value = self._data[key]
                except:
                    self._logger.debug('Sending "key not found"')
                    self._send('key not found')
                else:
                    self._logger.debug('Sending "ok"')
                    self._send('ok')
                    self._logger.debug('Sending value')
                    self._send(value)
                    
            elif action == 'del':
                try:
                    del self._data[key]
                except:
                    self._logger.debug('Sending "key not found"')
                    self._send('key not found')
                else:
                    self._logger.debug('Sending "ok"')
                    self._send('ok')
                
            elif action == 'size':
                self._send('ok')
                self._send(str(len(self._data)))

            elif action == 'keys':
                pickled = pickle.dumps(self._data.keys())
                self._send('ok')
                self._send(pickled)

            else:
                self._send('unknown action %s'%action)

            self._logger.debug('Closing')
            try:
                conn.shutdown(socket.SHUT_RDWR)
            except:
                self._logger.error('Failed to shutdown')
            conn.close()

            if action == 'stop':
                break

        self._logger.debug('Stopped')



def test1():
    """Test Timer class."""
    from time import sleep
    from datetime import timedelta
    from Timer import Timer
    timer = Timer()
    sleep(0.001)
    assert timer.get() > timedelta(seconds=0.001)
    sleep(0.001)
    assert timer.get() > timedelta(seconds=0.001)
    assert timer.getTotal() > timedelta(seconds=0.002)
test1()



"""Defines Timer class."""

import datetime

class Timer:
    """Times duration of code blocks."""

    def __init__(self):
        """Initialize the object, marking the current time."""
        self._started = datetime.datetime.now()
        self._previous = self._started

    def getTotal(self):
        """Return a :class:`datetime.timedelta` object representing
        total time elapsed since construction."""
        now = datetime.datetime.now()
        result = now - self._started
        return result

    def get(self):
        """Return a :class:`datetime.timedelta` object representing
        time elapsed since object construction, 
        or since last call to this function, whichever is more recent."""
        now = datetime.datetime.now()
        result = now - self._previous
        self._previous = now
        return result



"""Defines the Ring class.

The code, before slight variation in turn() method, was taken from:
  http://code.activestate.com/recipes/52246-implementing-a-circular-data-structure-using-lists
"""

class Ring:
    """Circular data structure implemented as a list."""

    def __init__(self, donor):
        """Initialize the ring with a donor list."""
        if not donor:
            raise 'Ring must have at least one element.'
        self._data = donor
        
    def __repr__(self):
        return repr(self._data)

    def __len__(self):
        return len(self._data)

    def __getitem__(self, index):
        return self._data[index]

    def turn(self):
        """Turn the ring for a single position.
        For example, [a, b, c, d] becomes [b, c, d, a].""" 
        first = self._data.pop(0)
        self._data.append(first)

    def first(self):
        """Return the first entry."""
        return self._data[0]

    def last(self):
        """Return the last entry."""
        return self._data[-1]



"""Simple socket communication for variable-length messages."""

import socket

class SocketTalk:
    """A simple layer of socket communication, implementing 
    send/receive messaging protocol where each (variable length) 
    message is prefixed with a (fixed length) header containing 
    the length of the remaining message data string. 

    Ideas for protocol and byte-handling are borrowed from:
       http://docs.python.org/howto/sockets.html.
    """

    # Define the header of every message string handed to the socket.
    HEADER_LENGTH = 16
    HEADER_FORMAT = '{{:{0}d}}'.format(HEADER_LENGTH)

    @staticmethod
    def pair():
        """Return a pair of connected SocketTalk peers."""
        s1, s2 = socket.socketpair()
        return SocketTalk(s1), SocketTalk(s2)

    @staticmethod
    def server(addr):
        """Return a SocketTalk server."""
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind(addr)
        sock.listen(1)
        conn, addr = sock.accept()
        talk = SocketTalk(conn)
        return talk

    @staticmethod
    def client(addr):
        """Return a SocketTalk client."""
        success = False
        while not success:
            try:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                sock.connect(addr)
                success = True
            except socket.error as err:
                sock.close()
        talk = SocketTalk(sock)
        return talk
        
    def __init__(self, sock, encode=True):
        """Initialize the object with a socket."""
        self._sock = sock
        self._encode = encode

    def put(self, message):
        """Send the given *message*.
        Return True if successful, False if not."""

        # First assemble the complete message string by prefixing the header.
        header = self.HEADER_FORMAT.format(len(message))
        full_message = header + message

        # Then send all bytes of the message.
        sent_count = 0
        while sent_count < len(full_message):
            try:
                msg = full_message[sent_count:]                
                msg = msg.encode() if self._encode else msg
                count = self._sock.send(msg)
            except socket.error as err:
                print('error')
                return False
            if count == 0:
                return False
            sent_count += count
        return True

    def get(self):
        """Receive a message.
        Return the message upon successful reception, or None upon failure."""

        # First retrieve all header bytes in order to extract
        # length of the message remainder.
        header = ''
        while len(header) < self.HEADER_LENGTH:
            chunk = self._sock.recv(self.HEADER_LENGTH - len(header))
            chunk = chunk.decode() if self._encode else chunk
            if chunk == '':
                return None
            header += chunk        
        length = int(header)

        # Then retrieve the remainder of the message.
        message = ''
        while len(message) < length:
            chunk = self._sock.recv(length - len(message))
            chunk = chunk.decode() if self._encode else chunk
            if chunk == '':
                return None
            message += chunk
        return message

    def close(self):
        self._sock.close()



def test1():
    """Test the SocketTalk peers."""
    from SocketTalk import SocketTalk
    talk1, talk2 = SocketTalk.pair()
    talk1.put('one')
    assert talk2.get() == 'one'
    talk2.put('two')
    assert talk1.get() == 'two'
    talk2.close()
    assert talk1.get() is None
    talk1.close()
test1()

def test2():
    """Test the SocketTalk client/server."""
    from threading import Thread
    from SocketTalk import SocketTalk

    addr = 'localhost', 50002
    def client():
        talk = SocketTalk.client(addr)
        talk.put('one')
        assert talk.get() == 'two'
        talk.close()

    def server():
        talk = SocketTalk.server(addr)
        assert talk.get() == 'one'
        talk.put('two')
        talk.close()

    Thread(target=client).start()
    Thread(target=server).start()
test2()




def test1():
    """Test UserInput."""
    from UserInput import user_input
    assert user_input('Name') == 'Bugs'
    assert user_input('Surname', default='Bunny') == 'Bunny'
    assert user_input('Bird', choices=('Tweety','Beaky')) == 'Tweety'
    assert user_input('Bird', choices=('Tweety','Beaky')) == 'Beaky'
    assert not user_input('empty input', empty_ok=True)
    assert user_input('last one') == 'The end.'



def test1():
    """Test Averager."""
    from Averager import Averager
    a = Averager(0)  # Same as Averager(1)
    assert len(a) == 0
    assert a.add(1.0) == 1.0  # 1.0 / 1
    assert len(a) == 1
    assert a.add(2.0) == 2.0  # 2.0 / 1
    assert len(a) == 1
    assert a.add(3.0) == 3.0  # 3.0 / 1
test1()

def test2():
    """Test Averager."""
    from Averager import Averager
    a = Averager(2)
    assert len(a) == 0
    assert a.add(1.0) == 1.0  # 1.0 / 1
    assert len(a) == 1
    assert a.add(2.0) == 1.5  # (1.0 + 2.0) / 2
    assert len(a) == 2
    assert a.add(3.0) == 2.5  # (2.0 + 3.0) / 2
    assert len(a) == 2
test2()



"""Useful string conversions and such."""

from os.path import join
from datetime import datetime

MICRO = '.%f'
TIME_FORMAT = '%Y-%m-%d %H:%M:%S' + MICRO

def string2time(text):
    """Return :class:`datetime.datetime` object from given string,
    or ``None`` if failed to translate."""
    length = len(TIME_FORMAT)
    result = None
    while length:
        try:
            result = datetime.strptime(text, TIME_FORMAT[:length])
        except:
            length -= 1
        else:
            break
    return result
    # The old function:
    #"""Return datetime object from given string.
    #Attempt to translate the string with microseconds, and if that
    #fails, translate it without microseconds."""
    #try:
    #    return datetime.strptime(text, TIME_FORMAT)
    #except:
    #    return datetime.strptime(text, TIME_FORMAT[:-len(MICRO)])

def time2string(tstamp, micro=True):
    """Given a :class:`datetime.datetime` object,
    return a formatted time string."""
    tformat = TIME_FORMAT if micro else TIME_FORMAT[:-len(MICRO)]
    return tstamp.strftime(tformat)

def time2levels(tstamp):
    """Given a :class:`datetime.datetime` object,
    return a list of directory levels (as strings).

    For example, given "2013-09-08 13:01:44",
    return ['2013', '09', '08', '13', '01'] 
    """
    return [tstamp.strftime(xx) for xx in ('%Y', '%m', '%d', '%H', '%M')]

def time2dir(tstamp):
    """Given a :class:`datetime.datetime` object,
    return a path assembled with :func:`os.path.join`
    for the levels."""
    result = ''
    for field in time2levels(tstamp):
        result = join(result, field)
    return result

FILENAME_FORMAT = '__%Y-%m-%d__%H:%M:%S:%f__'

def time2fname(tstamp, full=False):
    """Return full path to filename prefix (i.e. without dot extension)
    represented by given :class:`datetime.datetime` object."""
    result = tstamp.strftime(FILENAME_FORMAT)
    result = result if not full else join(time2dir(tstamp), result)
    return result



# -*- coding: utf-8 -*-

def test1():
    """Start from empty, set and get."""
    from Config import Config
    c = Config()
    c['NAME'] = 'Bender'
    c['SURNAME'] = 'Rodríguez'
    assert c['NAME'] == 'Bender'
    assert c['SURNAME'] == 'Rodríguez'
test1()

def test2():
    """Read from file and get."""
    from os.path import join, dirname, abspath
    from Config import Config
    fname = join(dirname(abspath(__file__)), 'simple.cfg')
    c = Config(fname)
    assert c['TOON'] == 'Bugs Bunny'
    assert c['QUOTE'] == 'The quick brown fox jumped over the lazy dog.'
test2()

def test3():
    """Read from file containing erroneous line, and get."""
    from os.path import join, dirname, abspath
    from Config import Config
    fname = join(dirname(abspath(__file__)), 'simple2.cfg')
    c = Config(fname)
    assert c['Kingdom'] == 'Animalia'
    assert c['Genus'] == 'Python'
test3()

def test4():
    """Read from file, get, save, load via constructor and load()."""
    from os.path import join, dirname, abspath
    from Config import Config
    fname1 = join(dirname(abspath(__file__)), 'simple.cfg')
    c1 = Config(fname1)
    fname2 = join(dirname(abspath(__file__)), 'test4.out')
    c1.save(fname2)
    c2 = Config(fname2)
    assert c2['TOON'] == 'Bugs Bunny'
    assert c2['QUOTE'] == 'The quick brown fox jumped over the lazy dog.'
    c3 = Config()
    c3.load(fname2)
    assert c3['TOON'] == 'Bugs Bunny'
    assert c3['QUOTE'] == 'The quick brown fox jumped over the lazy dog.'
test4()



from distutils.core import setup
from src import __version__

setup(
    name         = 'coils',
    version      = __version__,
    description  = 'A small library of generic codes written in 100% pure Python.',
    url          = 'http://vmlaker.github.io/coils',
    author       = 'Velimir Mlaker',
    author_email = 'velimir.mlaker@gmail.com',
    license      = 'MIT',
    #long_description = open('README.rst').read(),
    package_dir  = {'coils' : 'src'},
    packages     = ['coils'],
    classifiers  = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: Freeware',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
    )



def test1():
    """Test the MapSock client/server."""

    from threading import Thread
    from MapSock import MapSockServer, MapSockClient, MapSockRequest

    host = 'localhost'
    port = 50002

    # Start the server.
    server = MapSockServer(host, port)
    thread = Thread(target=server.run)
    thread.start()
    
    # Send set, get, stop.    
    client = MapSockClient(host, port)
    assert client.send(MapSockRequest('size')) == '0'
    assert client.send(MapSockRequest('set', 'tool', 'wrench')) == 'ok'
    assert client.send(MapSockRequest('set', 'fruit', 'apple')) == 'ok'
    assert client.send(MapSockRequest('set', 'animal', 'rabbit')) == 'ok'
    assert client.send(MapSockRequest('size')) == '3'
    assert client.send(MapSockRequest('get', 'fruit')) == 'apple'
    assert client.send(MapSockRequest('get', 'animal')) == 'rabbit'
    assert client.send(MapSockRequest('get', 'pizza')) == 'key not found'
    assert client.send(MapSockRequest('get', 'tool')) == 'wrench'
    assert client.send(MapSockRequest('del', 'fruit')) == 'ok'
    assert client.send(MapSockRequest('del', 'animal')) == 'ok'
    assert client.send(MapSockRequest('del', 'pizza')) == 'key not found'
    assert client.send(MapSockRequest('size')) == '1'
    assert client.send(MapSockRequest('stop')) == 'ok'

    # Join the server thread.
    thread.join()
test1()



"""A micro config utility,
loadable, saveable, one-line-per-entry config class."""

class Config(object):
    
    def __init__(self, fname=None):
        """Create config object, optionally load from file."""
        self._data = dict()
        if fname:
            self.load(fname)

    def load(self, fname):
        """Load config with contents of file."""
        with open(fname, 'r') as f:
            count = -1
            for line in f.readlines():
                count += 1
                line = line.strip()
                if not line:
                    continue
                if line[0] == '#':
                    continue
                key = line.split()[0]
                val = line[len(key):].lstrip()
                if not val:
                    print('Error line {0} file {1}'.format(count, fname))
                    continue
                self._data[key] = val

    def save(self, fname):
        """Save config to file."""
        with open(fname, 'w') as f:
            f.write(str(self))

    def items(self):
        """Return a copy of (key, value) pairs."""
        return self._data.items()

    def __setitem__(self, key, val):
        """Add entry using index operator."""
        self._data[key] = val
        
    def __getitem__(self, key):
        """Retrieve entry using index operator."""
        if key not in self._data:
            return None
        return self._data[key]

    def __str__(self):
        """Return string representation."""
        result = ''
        for key in sorted(self._data):
            if result:
                result += '\n'
            result += '{0} {1}'.format(key, self._data[key])
        return result



def test1():
    """Tick a simple ticker."""
    from RateTicker import RateTicker
    a = RateTicker((1,))
    assert a.tick() == (1.0,)
test1()

def test2():
    """Tick a three-period ticker."""
    from RateTicker import RateTicker
    a = RateTicker((1,2,5,))
    assert a.tick() == (1.0, 0.5, 0.2)
test2()



"""Defines Averager class."""

import collections

class Averager(object):
    """Keeps a running average with limited history."""

    def __init__(self, max_count):
        """Initialize the averager with maximum number of (latest) samples to keep."""
        self._max_count = max_count if max_count > 1 else 1  # Minimum is 1.
        self._data = collections.deque()

    def add(self, value):
        """Add a value, and return current average."""
        self._data.append(value)
        if len(self._data) > self._max_count:
            self._data.popleft()
        return sum(self._data)/len(self._data)

    def __len__(self):
        """Length operator."""
        return len(self._data)
